@@ -8,7 +8,7 @@ selection method.
 """
 
 #
-# Copyright (c) 2001 - 2015 The SCons Foundation
+# Copyright (c) 2001 - 2017 The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -30,7 +30,7 @@ selection method.
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "src/engine/SCons/Platform/posix.py rel_2.4.1:3453:73fefd3ea0b0 2015/11/09 03:25:05 bdbaddog"
+__revision__ = "src/engine/SCons/Platform/posix.py 74b2c53bc42290e911b334a6b44f187da698a668 2017/11/14 13:16:53 bdbaddog"
 
 import errno
 import os
@@ -48,16 +48,18 @@ exitvalmap = {
 }
 
 def escape(arg):
-    "escape shell special characters"
+    "escape shell special characters" 
     slash = '\\'
-    special = '"$()'
+    special = '"$'
 
     arg = arg.replace(slash, slash+slash)
     for c in special:
         arg = arg.replace(c, slash+c)
 
+    # print("ESCAPE RESULT: %s" % arg)
     return '"' + arg + '"'
 
+
 def exec_subprocess(l, env):
     proc = subprocess.Popen(l, env = env, close_fds = True)
     return proc.wait()
